/******************************************************************************
//                                                                           //
//  Project: Software Interface for the C32T12 Card                          //
//                                                                           //
//  CGC Instruments, Copyright  2003. Alle Rechte vorbehalten.              //
//                                                                           //
//  Definition for C32T12 DLL Routies                                        //
//                                                                           //
******************************************************************************/

#ifndef __C32T12_H
#define __C32T12_H

#define  STRICT
#include <windows.h>

#ifdef __cplusplus
extern "C" {
#endif


/* Counter status bits (R/W=read/write, R/O=read only): */
#define CNT32_GateCtrl  0x01 /* Gate Control: R/W, activates the counter gate by the software */
#define CNT32_MeasRun   0x02 /* Measurement Run: R/W, activates the measurement */
#define CNT32_InputNeg  0x04 /* Input Negative: R/W, inverts the input pulses, suitable for counting negative pulses */
#define CNT32_MeasSngl  0x08 /* Measurement Single: R/W, performs single measurement, the measurement stops after one gate activation */
#define CNT32_ResetCtrl 0x10 /* Reset Control: R/W, resets the counter data register and the internal logic */
#define CNT32_GateEn    0x20 /* Gate Enable: R/W, enables the gate input */
#define CNT32_MeasAct   0x40 /* Measurement Active: R/O, indicates a running measurement */
#define CNT32_GateAct   0x80 /* Gate Active: R/O, indicates an active gate */

/* General routines: */

/* Open the interface */
int FAR _export C32T12_Open();

/* Close the interface */
int FAR _export C32T12_Close();

/* Get error message for the specified ErrorCode */
const char * FAR _export C32T12_ErrorMessage (int ErrorCode);

/*
Check for presence of the C32T12 device
return 0 if present, -1 if device not found, ErrorCode on bus error */
int FAR _export C32T12_CheckPresence();


/* Counter routines: */

/*
Get counter data as 32-bit integer
Overflow indicates a possible counter overflow (Data MSB=1), Data may still be valid
Return 0 if successfull, ErrorCode on bus error
*/
int FAR _export CNT32_GetData (unsigned long & Data, bool & Overflow);

/*
Reset counter
Return 0 if successfull, ErrorCode on bus error
*/
int FAR _export CNT32_Reset();

/*
Get counter status as 8-bit integer
Return 0 if successfull, ErrorCode on bus error
*/
int FAR _export CNT32_GetStatus (BYTE & Status);

/*
Set counter status
Return 0 if successfull, ErrorCode on bus error
*/
int FAR _export CNT32_SetStatus (BYTE Status);

/*
Set counter status Count times
Return 0 if successfull, ErrorCode on bus error
*/
int FAR _export CNT32_ChangeStatus (BYTE * StatusBytes, unsigned Count);

/*
Flip bit(s) of counter status according to the BitMask
Return 0 if successfull, ErrorCode on bus error
*/
int FAR _export CNT32_FlipStatusBit (BYTE BitMask);


/* Timer routines: */

/*
Check the value Delay, round it to the nearest allowed value or increase by Delta steps
return ControlValue for DEL12_SetDelay */
WORD FAR _export TMR12_ConvertDelay (double & Delay, double ClockFrequency, int Delta = 0);

/* Set the delay according the ControlValue
return 0 if successfull, ErrorCode on bus error */
int FAR _export TMR12_SetDelay (WORD ControlValue, bool FallingSlope, int Channel);


#ifdef __cplusplus
}
#endif


#endif
