//----------------------------------------------------------------------------
//  Project C32T12Control
//  CGC Instruments
//  Copyright  2003. Alle Rechte vorbehalten.
//
//  SUBSYSTEM:    C32T12Control Application
//  FILE:         C32T12ControlDlgClient.cpp
//  AUTHOR:       Ivo Cermak
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TC32T12ControlDlgClient (TDialog).
//
//----------------------------------------------------------------------------

#include <owl/pch.h>

#include <stdio.h>

#include "C32T12ControlApp.h"
#include "C32T12ControlDlgClient.h"
#include "TestDialog.h"
#include "MeasureDialog.h"

#include <C32T12.h>


#define EV_WM_IFC_INIT\
	{WM_IFC_INIT, 0, (TAnyDispatcher)::v_Dispatch,\
	 (TMyPMF)v_Sig(&TMyClass::IfcInit)}

DEFINE_RESPONSE_TABLE1(TC32T12ControlDlgClient, TDialog)
	EV_WM_TIMER,
	EV_WM_IFC_INIT,
	EV_COMMAND(IDOK, CmOk),
	EV_COMMAND(IDCANCEL, CmCancel),
	EV_BN_CLICKED(IDC_MEAS_SNGL, BNMeasSingleClicked),
	EV_BN_CLICKED(IDC_MEAS_RUN, BNMeasRunClicked),
	EV_BN_CLICKED(IDC_IN_NEG, BNInputNegClicked),
	EV_BN_CLICKED(IDC_GATE_CTRL, BNGateCtrlClicked),
	EV_BN_CLICKED(IDC_GATE_EN, BNGateEnClicked),
	EV_BN_CLICKED(IDC_RESET, BNResetClicked),
	EV_BN_CLICKED(IDC_TEST, BNTestClicked),
	EV_BN_CLICKED(IDC_MEASURE, BNMeasureClicked),
	EV_EN_CHANGE(IDC_TMR_CLOCK, ENClockChange),
	EV_UDN_DELTAPOS(IDC_DELAY1_UPDOWN, UDNDelayCh1Deltapos),
	EV_UDN_DELTAPOS(IDC_DELAY2_UPDOWN, UDNDelayCh2Deltapos),
	EV_EN_KILLFOCUS(IDC_DELAY1, UpdateCh1),
	EV_EN_KILLFOCUS(IDC_DELAY2, UpdateCh2),
	EV_BN_CLICKED(IDC_SW_TRIG1, BNSwTriggerCh1Clicked),
	EV_BN_CLICKED(IDC_SW_TRIG2, BNSwTriggerCh2Clicked),
	EV_BN_CLICKED(IDC_RISSING1, UpdateCh1),
	EV_BN_CLICKED(IDC_TRAILING1, UpdateCh1),
	EV_BN_CLICKED(IDC_RISSING2, UpdateCh2),
	EV_BN_CLICKED(IDC_TRAILING2, UpdateCh2),
END_RESPONSE_TABLE;


enum { DataTimer };
enum { DataTimeout = 50 };


TC32T12ControlDlgClient::TC32T12ControlDlgClient (TWindow * parent, TResId resId, TModule * module) :
	TDialog (parent, resId, module)
	{ isOpen = isPresent = false; Counts = -1; GateTime = 1000; Running = false; ClockFrequency = -1; }

TC32T12ControlDlgClient::~TC32T12ControlDlgClient()
	{ Destroy(); }

bool TC32T12ControlDlgClient::IdleAction (long idleCount) {
	if (Running) return TDialog::IdleAction (idleCount);
	return false;
	}

void TC32T12ControlDlgClient::SetupWindow() {
	TDialog::SetupWindow();
	SendDlgItemMessage (IDC_PRESENT, BM_SETCHECK, BF_GRAYED);
	SendDlgItemMessage (IDC_DELAY1_UPDOWN, UDM_SETRANGE, 0, MkParam2 (-0x7FFFFFFF, 0x7FFFFFFF));
	SendDlgItemMessage (IDC_DELAY2_UPDOWN, UDM_SETRANGE, 0, MkParam2 (-0x7FFFFFFF, 0x7FFFFFFF));
	SendDlgItemMessage (IDC_DELAY1_UPDOWN, UDM_SETPOS,   0, 0);
	SendDlgItemMessage (IDC_DELAY2_UPDOWN, UDM_SETPOS,   0, 0);
	SendDlgItemMessage (IDC_RISSING1, BM_SETCHECK, BF_CHECKED);
	SendDlgItemMessage (IDC_RISSING2, BM_SETCHECK, BF_CHECKED);
	SetTimer (DataTimer, DataTimeout);
	PostMessage (WM_IFC_INIT);
	}

void TC32T12ControlDlgClient::IfcInit() {
	IfcOpen();
	if (isOpen) isPresent = CheckPresence();
	SendDlgItemMessage (IDC_PRESENT, BM_SETCHECK, !isOpen ? BF_GRAYED : isPresent ? BF_CHECKED : BF_UNCHECKED);
	CntUpdate(); TmrUpdate();
	}

bool TC32T12ControlDlgClient::CanClose() {
	if (!TDialog::CanClose()) return false;
	KillTimer (DataTimer); IfcClose(); return true;
	}


void TC32T12ControlDlgClient::IfcOpen() {
	const int Result = C32T12_Open(); CheckStatus (Result);
	isOpen = (Result == 0);
	}

void TC32T12ControlDlgClient::IfcClose() {
	if (isOpen) { const int Result = C32T12_Close(); CheckStatus (Result); isOpen = isPresent = false; }
	SendDlgItemMessage (IDC_PRESENT, BM_SETCHECK, BF_GRAYED);
	}


void TC32T12ControlDlgClient::CntUpdate()
	{ UpdateData(); UpdateStatus(); }

void TC32T12ControlDlgClient::TmrUpdate()	{
	ENClockChange(); UpdateCh1(); UpdateCh2();
	::EnableWindow (GetDlgItem (IDC_RISSING1), isPresent);
	::EnableWindow (GetDlgItem (IDC_RISSING2), isPresent);
	::EnableWindow (GetDlgItem (IDC_TRAILING1), isPresent);
	::EnableWindow (GetDlgItem (IDC_TRAILING2), isPresent);
	::EnableWindow (GetDlgItem (IDC_SW_TRIG1), isPresent);
	::EnableWindow (GetDlgItem (IDC_SW_TRIG2), isPresent);
	}


bool TC32T12ControlDlgClient::CheckStatus (int Status)	{
	const char * const Message = C32T12_ErrorMessage (Status); const bool Result = (Status == 0);
	if (!Result) {
		char TitleString [100]; GetModule()->LoadString (IDS_IFC_ERROR, TitleString, sizeof (TitleString));
		if (MessageBox (Message, TitleString, MB_OKCANCEL) != IDOK) PostQuitMessage (-1);
		}
	return Result;
	}

bool TC32T12ControlDlgClient::CheckPresence() {
	const Result = C32T12_CheckPresence(); const bool Present = (Result == 0);
	if (Result > 0) CheckStatus (Result);
	return Present;
	}


void TC32T12ControlDlgClient::UpdateData() {
	if (!isPresent) {
		SetDlgItemText (IDC_DATA, NULL);
		SendDlgItemMessage (IDC_OVERFLOW, BM_SETCHECK, BF_GRAYED);
		Counts--; return;
		}
	unsigned long NewCounts; bool Overflow; const Result = CNT32_GetData (NewCounts, Overflow); if (Result) CheckStatus (Result);
	if (Counts == NewCounts) return;
	Counts = NewCounts;
	SetDlgItemInt (IDC_DATA, NewCounts, false);
	SendDlgItemMessage (IDC_OVERFLOW, BM_SETCHECK, (NewCounts & (1 << 31)) ? BF_CHECKED : BF_UNCHECKED);
	}

void TC32T12ControlDlgClient::UpdateStatus() {
	static int LastStatus = -1; int Status;
	if (!isPresent) Status = -2;
	else { BYTE newStatus; const Result = CNT32_GetStatus (newStatus); if (Result) CheckStatus (Result); Status = newStatus; }
	if (LastStatus == Status) return;
	LastStatus = Status;
	SendDlgItemMessage (IDC_GATE_ACT,  BM_SETCHECK, GetButtonFlag (Status, CNT32_GateAct ));
	SendDlgItemMessage (IDC_GATE_EN,   BM_SETCHECK, GetButtonFlag (Status, CNT32_GateEn  ));
	SendDlgItemMessage (IDC_GATE_CTRL, BM_SETCHECK, GetButtonFlag (Status, CNT32_GateCtrl));
	SendDlgItemMessage (IDC_MEAS_ACT,  BM_SETCHECK, GetButtonFlag (Status, CNT32_MeasAct ));
	SendDlgItemMessage (IDC_MEAS_RUN,  BM_SETCHECK, GetButtonFlag (Status, CNT32_MeasRun ));
	SendDlgItemMessage (IDC_MEAS_SNGL, BM_SETCHECK, GetButtonFlag (Status, CNT32_MeasSngl));
	SendDlgItemMessage (IDC_IN_NEG,    BM_SETCHECK, GetButtonFlag (Status, CNT32_InputNeg));
	::EnableWindow (GetDlgItem (IDC_RESET),   isPresent);
	::EnableWindow (GetDlgItem (IDC_MEASURE), isPresent);
	::EnableWindow (GetDlgItem (IDC_TEST),    isPresent);
	}

void TC32T12ControlDlgClient::FlipStatusBit (BYTE Mask) {
	if (!isPresent) return;
	const Result = CNT32_FlipStatusBit (Mask); if (Result) CheckStatus (Result);
	UpdateStatus();
	}


void TC32T12ControlDlgClient::EvTimer (uint TimerId) {
	switch (TimerId) {
		case DataTimer: CntUpdate(); break;
		default: TDialog::EvTimer (TimerId);
		}
	}

void TC32T12ControlDlgClient::BNMeasSingleClicked()
	{ FlipStatusBit (CNT32_MeasSngl); }

void TC32T12ControlDlgClient::BNMeasRunClicked()
	{ FlipStatusBit (CNT32_MeasRun); }

void TC32T12ControlDlgClient::BNInputNegClicked()
	{ FlipStatusBit (CNT32_InputNeg); }

void TC32T12ControlDlgClient::BNGateEnClicked()
	{ FlipStatusBit (CNT32_GateEn); }

void TC32T12ControlDlgClient::BNGateCtrlClicked()
	{ FlipStatusBit (CNT32_GateCtrl); }


void TC32T12ControlDlgClient::BNResetClicked() {
	if (!isPresent) return;
	const Result = CNT32_Reset(); if (Result) CheckStatus (Result);
	UpdateStatus();
	}

void TC32T12ControlDlgClient::BNTestClicked() {
	if (!isPresent) return;
	Running = true; KillTimer (DataTimer);
	TTestDialog (this, IDD_TEST).Execute();
	Running = false; SetTimer (DataTimer, DataTimeout);
	}

void TC32T12ControlDlgClient::BNMeasureClicked() {
	if (!isPresent) return;
	Running = true; KillTimer (DataTimer);
	TMeasureDialog (this, IDD_MEASURE).Execute();
	Running = false; SetTimer (DataTimer, DataTimeout);
	}


void TC32T12ControlDlgClient::SetDelay (WORD CtrlValue, bool TrailingSlope, int Channel)
	{ const int Result = TMR12_SetDelay (CtrlValue, TrailingSlope, Channel); if (Result > 0) CheckStatus (Result); }


WORD TC32T12ControlDlgClient::ConvertDelay (double & Delay, int Delta)
	{ return TMR12_ConvertDelay (Delay, ClockFrequency*1e6, Delta); }


void TC32T12ControlDlgClient::ProcessDelay (int Channel, int Delta, bool SwTrigger) {
	char Buffer [20]; if (!GetDlgItemText (Channel==1 ? IDC_DELAY1 : IDC_DELAY2, Buffer, sizeof (Buffer))) *Buffer = '\0';
	double oldDelay = 0; sscanf (Buffer, "%lf", &oldDelay);
	double newDelay = oldDelay;	const WORD CtrlValue = ConvertDelay (newDelay, Delta);
	sprintf (Buffer, "%.4lE", newDelay ); SetDlgItemText (Channel==1 ? IDC_DELAY1   : IDC_DELAY2,   Buffer);
	sprintf (Buffer, "%04hX", CtrlValue); SetDlgItemText (Channel==1 ? IDC_CTRLVAL1 : IDC_CTRLVAL2, Buffer);
	if (!isPresent) return;
	const bool TrailingSlope = (SendDlgItemMessage (Channel==1 ? IDC_TRAILING1 : IDC_TRAILING2, BM_GETCHECK) == 1);
	if (SwTrigger) SetDelay (CtrlValue, !TrailingSlope, Channel);
	SetDelay (CtrlValue, TrailingSlope, Channel);
	}


void TC32T12ControlDlgClient::UpdateCh1()
	{ ProcessDelay (1); }

void TC32T12ControlDlgClient::UpdateCh2()
	{ ProcessDelay (2); }

bool TC32T12ControlDlgClient::UDNDelayCh1Deltapos (TNmUpDown & UpDn)
	{ ProcessDelay (1, UpDn.iDelta); SendDlgItemMessage (IDC_DELAY1_UPDOWN, UDM_SETPOS, 0, 0); return true; }

bool TC32T12ControlDlgClient::UDNDelayCh2Deltapos (TNmUpDown & UpDn)
	{ ProcessDelay (2, UpDn.iDelta); SendDlgItemMessage (IDC_DELAY2_UPDOWN, UDM_SETPOS, 0, 0); return true; }

void TC32T12ControlDlgClient::ENClockChange() {
	char Buffer [20]; if (!GetDlgItemText (IDC_TMR_CLOCK, Buffer, sizeof (Buffer))) *Buffer = '\0';
	double NewClockFrequency = -10; sscanf (Buffer, "%lf", &NewClockFrequency);
	if (ClockFrequency != NewClockFrequency) {
		if (NewClockFrequency < 0.01) NewClockFrequency = 10;
		ClockFrequency = NewClockFrequency;
		sprintf (Buffer, "%.2lf", ClockFrequency);
		SetDlgItemText (IDC_TMR_CLOCK, Buffer);
		}
	UpdateCh1(); UpdateCh2();
	}

void TC32T12ControlDlgClient::BNSwTriggerCh1Clicked()
	{ ProcessDelay (1, 0, true); }

void TC32T12ControlDlgClient::BNSwTriggerCh2Clicked()
	{ ProcessDelay (2, 0, true); }

void TC32T12ControlDlgClient::CmOk() {
	const THandle FocusHandle = GetFocus();
	if      (FocusHandle == GetDlgItem (IDC_DELAY1)) UpdateCh1();
	else if (FocusHandle == GetDlgItem (IDC_DELAY2)) UpdateCh2();
	else if (FocusHandle == GetDlgItem (IDC_TMR_CLOCK)) ENClockChange();
	}

