//----------------------------------------------------------------------------
//  Project C32T12Control
//  CGC Instruments
//  Copyright  2003. Alle Rechte vorbehalten.
//
//  SUBSYSTEM:    C32T12Control.apx Application
//  FILE:         MeasureDialog.cpp
//  AUTHOR:       Ivo Cermak
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TMeasureDialog (TDialog).
//
//----------------------------------------------------------------------------
#include <owl/pch.h>

#include "MeasureDialog.h"

#include "C32T12.h"


//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(TMeasureDialog, TDialog)
//{{TMeasureDialogRSP_TBL_BEGIN}}
  EV_WM_TIMER,
//{{TMeasureDialogRSP_TBL_END}}
END_RESPONSE_TABLE;


//{{TMeasureDialog Implementation}}

enum { DataTimer, StopTimer, GateTimer };
enum { DataTimeout = 50, StopTimeout = DataTimeout*3/2 };


TMeasureDialog::TMeasureDialog (TC32T12ControlDlgClient * parent, TResId resId, TModule * module) : TMeasureBase (parent, resId, module)
	{ }

TMeasureDialog::~TMeasureDialog()
	{ StopMeasurement(); Destroy(); }


void TMeasureDialog::InitMeasurement() {
	BYTE Buffer [4];
	Buffer [0] = OrgStatus | BYTE (CNT32_ResetCtrl);
	Buffer [1] = OrgStatus;
	Buffer [2] = OrgStatus | BYTE (CNT32_MeasRun);
	Buffer [3] = OrgStatus | BYTE (CNT32_MeasRun | CNT32_GateCtrl);
	ChangeStatus (Buffer, 4);
	}

void TMeasureDialog::StopMeasurement()
	{ SetStatus (OrgStatus); }

void TMeasureDialog::RunMeasurement() {
	StopMeasurement(); IfcParent->CntUpdate();
	IfcParent->GateTime = SendDlgItemMessage (IDC_GATE_UPDOWN, UDM_GETPOS);
	if (IfcParent->GateTime < 1) IfcParent->GateTime = 1; else if (IfcParent->GateTime > UD_MAXVAL) IfcParent->GateTime = UD_MAXVAL;
	SetTimer (GateTimer, IfcParent->GateTime); InitMeasurement();
	if (IfcParent->GateTime < StopTimeout) { KillTimer (DataTimer); KillTimer (StopTimer); }
	else { SetTimer (DataTimer, DataTimeout); SetTimer (StopTimer, IfcParent->GateTime - DataTimeout); }
	}

void TMeasureDialog::SetupWindow() {
	TMeasureBase::SetupWindow();
	SendDlgItemMessage (IDC_GATE_UPDOWN, UDM_SETRANGE, 0, MkParam2 (UD_MAXVAL, 1));
	SendDlgItemMessage (IDC_GATE_UPDOWN, UDM_SETPOS,   0, MkParam2 (IfcParent->GateTime, 0));
	BYTE Status; GetStatus (Status); OrgStatus = (Status & BYTE (CNT32_InputNeg | CNT32_MeasSngl));
	RunMeasurement();
	}

void TMeasureDialog::EvTimer (uint TimerId) {
	switch (TimerId) {
		case DataTimer: IfcParent->CntUpdate(); break;
		case StopTimer: KillTimer (DataTimer);  break;
		case GateTimer: RunMeasurement(); SetDlgItemInt (IDC_COUNTS, IfcParent->Counts, false); break;
		default: TDialog::EvTimer (TimerId);
		}
	}

